      subroutine vvvmd(iv,g,u1,p1,u2,p2,u,mat)
c----------------------------------------------------------------------c
c Returns a matrix element array for the triple vector-boson coupling
c
c Input: 
c     iv = 1, for u1 being 1st and u2 the 2nd particle in the vertex,
c     iv = 2, for u1 being 1st and u2 the 3rd particle in the vertex,
c     iv = 3, for u1 being 2nd and u2 the 1st particle in the vertex,
c     iv = 4, for u1 being 2nd and u2 the 3rd particle in the vertex,
c     iv = 5, for u1 being 3rd and u2 the 1st particle in the vertex,
c     iv = 6, for u1 being 3rd and u2 the 2nd particle in the vertex,
c     p1,p2 - momenta,
c     g - a triple-boson coupling.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 2 May, 2007; modified on 6/14 May (interface changed)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: iv
      complex(dp) :: g,mat
      complex(dp), dimension(0:3) :: u1,u2,u,ev
      real(dp), dimension(0:3) :: p1,p2,p

      p=-p1-p2

      if (iv == 1) then

           ev=g*((u1(0)*(p2(0)-p(0))
     &           -u1(1)*(p2(1)-p(1))
     &           -u1(2)*(p2(2)-p(2))
     &           -u1(3)*(p2(3)-p(3)))*u2
     &          +(u2(0)*(p(0)-p1(0))
     &           -u2(1)*(p(1)-p1(1))
     &           -u2(2)*(p(2)-p1(2))
     &           -u2(3)*(p(3)-p1(3)))*u1
     &          +(u1(0)*u2(0)
     &           -u1(1)*u2(1)
     &           -u1(2)*u2(2)
     &           -u1(3)*u2(3))*(p1-p2))
           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else if (iv == 2) then

           ev=g*((u1(0)*(p(0)-p2(0))
     &           -u1(1)*(p(1)-p2(1))
     &           -u1(2)*(p(2)-p2(2))
     &           -u1(3)*(p(3)-p2(3)))*u2
     &          +(u2(0)*(p1(0)-p(0))
     &           -u2(1)*(p1(1)-p(1))
     &           -u2(2)*(p1(2)-p(2))
     &           -u2(3)*(p1(3)-p(3)))*u1
     &          +(u1(0)*u2(0)
     &           -u1(1)*u2(1)
     &           -u1(2)*u2(2)
     &           -u1(3)*u2(3))*(p2-p1))
           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else if (iv == 3) then

           ev=g*((u2(0)*(p1(0)-p(0))
     &           -u2(1)*(p1(1)-p(1))
     &           -u2(2)*(p1(2)-p(2))
     &           -u2(3)*(p1(3)-p(3)))*u1
     &          +(u1(0)*(p(0)-p2(0))
     &           -u1(1)*(p(1)-p2(1))
     &           -u1(2)*(p(2)-p2(2))
     &           -u1(3)*(p(3)-p2(3)))*u2
     &          +(u2(0)*u1(0)
     &           -u2(1)*u1(1)
     &           -u2(2)*u1(2)
     &           -u2(3)*u1(3))*(p2-p1))

           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else if (iv == 4) then

           ev=g*((u2(0)*(p(0)-p1(0))
     &           -u2(1)*(p(1)-p1(1))
     &           -u2(2)*(p(2)-p1(2))
     &           -u2(3)*(p(3)-p1(3)))*u1
     &          +(u1(0)*(p2(0)-p(0))
     &           -u1(1)*(p2(1)-p(1))
     &           -u1(2)*(p2(2)-p(2))
     &           -u1(3)*(p2(3)-p(3)))*u2
     &          +(u1(0)*u2(0)
     &           -u1(1)*u2(1)
     &           -u1(2)*u2(2)
     &           -u1(3)*u2(3))*(p1-p2))
           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else if (iv == 5) then

           ev=g*((u2(0)*(p(0)-p1(0))
     &           -u2(1)*(p(1)-p1(1))
     &           -u2(2)*(p(2)-p1(2))
     &           -u2(3)*(p(3)-p1(3)))*u1
     &          +(u1(0)*(p2(0)-p(0))
     &           -u1(1)*(p2(1)-p(1))
     &           -u1(2)*(p2(2)-p(2))
     &           -u1(3)*(p2(3)-p(3)))*u2
     &          +(u1(0)*u2(0)
     &           -u1(1)*u2(1)
     &           -u1(2)*u2(2)
     &           -u1(3)*u2(3))*(p1-p2))
           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else if (iv == 6) then

           ev=g*((u2(0)*(p1(0)-p(0))
     &           -u2(1)*(p1(1)-p(1))
     &           -u2(2)*(p1(2)-p(2))
     &           -u2(3)*(p1(3)-p(3)))*u1
     &          +(u1(0)*(p(0)-p2(0))
     &           -u1(1)*(p(1)-p2(1))
     &           -u1(2)*(p(2)-p2(2))
     &           -u1(3)*(p(3)-p2(3)))*u2
     &          +(u1(0)*u2(0)
     &           -u1(1)*u2(1)
     &           -u1(2)*u2(2)
     &           -u1(3)*u2(3))*(p2-p1))
           mat=ev(0)*u(0)-ev(1)*u(1)-ev(2)*u(2)-ev(3)*u(3)

      else
       write(*,*)'vvvmd.f: No such option, iv=',iv
      end if

      end subroutine vvvmd
