      subroutine readpart(process)
c----------------------------------------------------------------------c
c Reads particles from process
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 9 February, 2006; modified 12 October, 2006 (particle reading improved)
c                            6 November, 2006 (wrong restoration of old
c                                              conventions removed),
c                            12 May, 2007 (mass and widths assignments
c                                          added)
c                            18 October, 2007 (MC summing over 
c                                              polarizations implemented)
c                            14 April, 2008 (determination of the 
c                            statistical factor for identical particles
c                            in the final state corrected)
c----------------------------------------------------------------------c

      use maxvals
      use mytypes
      use extpart
      use vertices

      implicit none

      character(80) :: process,proc,cline

      character(1), parameter :: space=' ',separ='>'
      integer :: i,j,k,l,n,blad,ipmod,iprt(mxprt,5),
     &           allmod,inmod,ii,difp,ifrmn,idscrd
      character(10) :: cprt(mxprt,4)
      character(1) :: lprt(mxprt)

      integer, allocatable :: idpf(:)
      character(3), allocatable :: expd(:)
      character(9) :: prtcl(4)

      proc=adjustl(process)

      j=0
      l=index(proc,separ)

      if (l == 0) then
       write(*,*)'Wrong separator, or process not entered.'
       write(*,*)'Please, try again!'
       stop
      end if


c Read initial state particles

      do 
       n=index(proc,space)
       j=j+1
       expc(j)=proc(1:n-1)
       f(j)%o=-1
       proc=adjustl(proc(n:l))
       if (proc(2:2) == separ) exit
      end do

c No of the initial state particles

      nisp=j

c Read final state 

      k=l+2

      proc=adjustl(process(l+1:))

      do 
       n=index(proc,space)
       if (n > 1) then
        j=j+1
        f(j)%o=1
        expc(j)=proc(1:n-1)
       else
        exit
       end if
       proc=adjustl(proc(n:))
      end do

c No of the external particles

      next=j

c No of identical particles in the final state

      allocate (idpf(next-nisp),stat=blad)
      allocate (expd(next-nisp),stat=blad)

      idpf=1
      difp=1
      expd(difp)=expc(nisp+1)

      do j=nisp+2,next
       do ii=difp,1,-1
        if (expc(j) == expd(ii)) then
         idpf(ii)=idpf(ii)+1
         exit
        else
         if (ii == 1) then
          difp=difp+1
          expd(difp)=expc(j)
         end if
        end if
       end do
      end do

      nidp=1

      do j=1,difp
       do ii=1,idpf(j)
        nidp=nidp*ii
       end do
      end do

      deallocate (idpf,expd,stat=blad)

c      open(66,file='modpart.dat',access='direct',form='formatted',
c     &                                                     recl=80)
      open(unit=66,file='modpart.dat',status='old',action='read')

c Read comments

      do 
       read(66,'(a80)') cline
       if (index(cline,'Begin input') > 0 ) exit
      end do

c Read particles

      ipmod=1

      do 
       read(66,'(4(a9),5(i5),a1)',end=600) cprt(ipmod,:),
     &                                    iprt(ipmod,:),lprt(ipmod)
       ipmod=ipmod+1
      end do

 600  close(66)

c Check if all the external particles belong to the model

      allmod=1

      do j=1,next
       inmod=0
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         inmod=1
         exit
        end if
       end do
       allmod=allmod*inmod
       if (allmod == 0) then
        write(*,*) trim(expc(j)),' is not a particle of the model'
        stop 
       end if
      end do

      nmcs=0
      nhip=1

      do j=1,next
       f(j)%e=1
       allocate (f(j)%ix(f(j)%e),stat=blad)
       allocate (f(j)%ip(0),stat=blad)
       f(j)%ix(f(j)%e)=j
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         expl(j)=lprt(i)
         if (trim(expc(j)) == trim(cprt(i,1))
     &      .and. trim(expc(j)) == trim(cprt(i,2))) then  ! neutral boson
          if (f(j)%o == -1) then
           f(j)%i=-1
          else
           f(j)%i=1
          end if
         else
          if (trim(expc(j)) == trim(cprt(i,1))) then
           f(j)%i=1
          else
           f(j)%i=-1
          end if
         end if
         f(j)%j=iprt(i,5)
         f(j)%f=iprt(i,1)
         f(j)%g=iprt(i,2)
         f(j)%n=iprt(i,4)
         f(j)%ct=iprt(i,3)
         f(j)%m=trim(cprt(i,3))
         f(j)%w=trim(cprt(i,4))
        end if
       end do
       if (f(j)%o == -1) then
        nhip=nhip*f(j)%n
       end if
       if (imcs == 1) then
        if (f(j)%n > 1) then
         nmcs=nmcs+1
         f(j)%mc=nmcs
         f(j)%n=1
        else
         f(j)%mc=0
        end if
       else
        f(j)%mc=0
       end if
      end do

c Define the SM vertices, using the notation
c f = 1-neutrino, 2-charged lepton, 3-up quark, 4-down quark;
c b = 0-Higgs, 1-photon, 2-Z, 3-W, 4-gluon.

c fermion-fermion-boson vertices, ffb and their couplings

c      ffb(1,:)=(/  1, -1, 21/)       !  d d~ g
c      gffb(1)='gqq'
c      ffb(2,:)=(/  2, -2, 21/)       !  u u~ g
c      gffb(2)='gqq'
c      ffb(3,:)=(/  3, -3, 21/)       !  s s~ g
c      gffb(3)='gqq'
c      ffb(4,:)=(/  4, -4, 21/)       !  c c~ g
c      gffb(4)='gqq'
c      ffb(5,:)=(/  5, -5, 21/)       !  b b~ g
c      gffb(5)='gqq'
c      ffb(6,:)=(/  6, -6, 21/)       !  t t~ g
c      gffb(6)='gqq'

c      ffb(7,:)=(/  1, -1, 22/)       !  d d~ a
c      gffb(7)='add'
c      ffb(8,:)=(/  2, -2, 22/)       !  u u~ a
c      gffb(8)='auu'
c      ffb(9,:)=(/  3, -3, 22/)       !  s s~ a
c      gffb(9)='add'
c      ffb(10,:)=(/  4, -4, 22/)      !  c c~ a
c      gffb(10)='auu'
c      ffb(11,:)=(/  5, -5, 22/)      !  b b~ a
c      gffb(11)='add'
c      ffb(12,:)=(/  6, -6, 22/)      !  t t~ a
c      gffb(12)='auu'

c      ffb( 13,:)=(/ 11,-11, 22/)     !  e- e+ a
c      gffb(13)='all'
c      ffb( 14,:)=(/ 13,-13, 22/)     !  mu- mu+ a
c      gffb(14)='all'
c      ffb( 15,:)=(/ 15,-15, 22/)     !  ta- ta+ a
c      gffb(15)='all'


c      ffb(16,:)=(/  1, -1, 23/)      !  d d~ z
c      gffb(16)='zdd'
c      ffb(17,:)=(/  2, -2, 23/)      !  u u~ z
c      gffb(17)='zuu'
c      ffb(18,:)=(/  3, -3, 23/)      !  s s~ z
c      gffb(18)='zdd' 
c      ffb(19,:)=(/  4, -4, 23/)      !  c c~ z
c      gffb(19)='zuu'
c      ffb(20,:)=(/  5, -5, 23/)      !  b b~ z
c      gffb(20)='zdd'
c      ffb(21,:)=(/  6, -6, 23/)      !  t t~ z
c      gffb(21)='zuu' 


c      ffb(22,:)=(/ 11,-11, 23/)      !  e- e+ z
c      gffb(22)='zll'
c      ffb(23,:)=(/ 12,-12, 23/)      !  ve ve~ z
c      gffb(23)='znn'
c      ffb(24,:)=(/ 13,-13, 23/)      !  mu- mu+ z
c      gffb(24)='zll'
c      ffb(25,:)=(/ 14,-14, 23/)      !  vm vm~ z
c      gffb(25)='znn'
c      ffb(26,:)=(/ 15,-15, 23/)      !  ta- ta+ z
c      gffb(26)='zll'
c      ffb(27,:)=(/ 16,-16, 23/)      !  vt vt~ z
c      gffb(27)='znn'


c      ffb(28,:)=(/  1, -2,-24/)      !  d u~ w-
c      gffb(28)='wff'
c      ffb(29,:)=(/  2, -1, 24/)      !  u d~ w+
c      gffb(29)='wff'
c      ffb(30,:)=(/  3, -4,-24/)      !  s c~ w-
c      gffb(30)='wff'
c      ffb(31,:)=(/  4, -3, 24/)      !  c s~ w+
c      gffb(31)='wff'
c      ffb(32,:)=(/  5, -6,-24/)      !  b t~ w-
c      gffb(32)='wff'
c      ffb(33,:)=(/  6, -5, 24/)      !  t b~ w+
c      gffb(33)='wff'

c      ffb(34,:)=(/ 11,-12,-24/)      !  e- ve~ w-
c      gffb(34)='wff'
c      ffb(35,:)=(/ 12,-11, 24/)      !  ve e+ w+
c      gffb(35)='wff'
c      ffb(36,:)=(/ 13,-14,-24/)      !  mu- vm~ w-
c      gffb(36)='wff'
c      ffb(37,:)=(/ 14,-13, 24/)      !  vm mu+ w+
c      gffb(37)='wff'
c      ffb(38,:)=(/ 15,-16,-24/)      !  ta- vt~ w-
c      gffb(38)='wff'
c      ffb(39,:)=(/ 16,-15, 24/)      !  vt ta+ w+
c      gffb(39)='wff'

c      ffb(40,:)=(/  1, -1, 25/)      !  d d~ h
c      ffb(40,:)=0
c      gffb(40)='hdd'
c      ffb(41,:)=(/  2, -2, 25/)      !  u u~ h
c      ffb(41,:)=0
c      gffb(41)='huu'
c      ffb(42,:)=(/  3, -3, 25/)      !  s s~ h
c      ffb(42,:)=0
c      gffb(42)='hss' 
c      ffb(43,:)=(/  4, -4, 25/)      !  c c~ h
c      gffb(43)='hcc'
c      ffb(44,:)=(/  5, -5, 25/)      !  b b~ h
c      gffb(44)='hbb'
c      ffb(45,:)=(/  6, -6, 25/)      !  t t~ h
c      gffb(45)='htt' 

c      ffb(46,:)=(/ 11,-11, 25/)      !  e- e+ h
c      ffb(46,:)=0
c      gffb(46)='hee'
c      ffb(47,:)=(/ 13,-13, 25/)      !  mu- mu+ h
c      ffb(47,:)=0
c      gffb(47)='hmm'
c      ffb(48,:)=(/ 15,-15, 25/)      !  ta- ta+ h
c      gffb(48)='hll'

c triple boson vertices, bbb (all the particles are incoming to the vertex)

c      bbb( 1,:)=(/ 21, 21, 21/)       !  g g g
c      gbbb(1)='ggg'

c      bbb( 2,:)=(/ 24,-24, 22/)       !  w+ w- a
c      gbbb(2)='aww'
c      bbb( 3,:)=(/ 24,-24, 23/)       !  w+ w- z
c      gbbb(3)='zww'
c      bbb( 4,:)=(/ 24,-24, 25/)       !  w+ w- h
c      gbbb(4)='hww'
c      bbb( 5,:)=(/ 23, 23, 25/)       !  z z h
c      gbbb(5)='hzz'
c      bbb( 6,:)=(/ 25, 25, 25/)       !  h h h
c      gbbb(6)='hhh'

c quartic boson vertices, bbbb

c      bbbb( 1,:)=(/ 21, 21, 21, 21/)   !  g g g g
c      gbbbb(1)='gggg'

c      bbbb( 2,:)=(/ 24,-24, 22, 22/)   !  w+ w- a a
c      gbbbb(2)='aaww'
c      bbbb( 3,:)=(/ 24,-24, 23, 23/)   !  w+ w- z z
c      gbbbb(3)='zzww'
c      bbbb( 4,:)=(/ 24,-24, 22, 23/)   !  w+ w- a z
c      gbbbb(4)='azww'
c      bbbb( 5,:)=(/ 24, 24,-24,-24/)   !  w+ w+ w- w-
c      gbbbb(5)='wwww'
c      bbbb( 6,:)=(/ 23, 23, 25, 25/)   !  z z h h
c      gbbbb(6)='hhzz'
c      bbbb( 7,:)=(/ 24,-24, 25, 25/)   !  w+ w- h h
c      gbbbb(7)='hhww'
c      bbbb( 8,:)=(/ 25, 25, 25, 25/)   !  h h h h
c      gbbbb(8)='hhhh'

ckk>
c No Higgs exchange:
c      ffb(40:48,:)=0
c      bbb(4:6,:)=0
c      bbbb(6:8,:)=0
ckk<

c Read fermion-fermion-boson vertices of the SM and the corresponding couplings

      open(unit=66,file='vertices.dat',status='old',action='read')

      nffba=0
      nbbba=0
      nbbbba=0

      do 
 1001  idscrd=0
       read(66,'(a80)',end=700) cline
       if (cline(1:1) == '#') then
       else
        cline=adjustl(cline)
        j=0
        l=index(cline,':')
        if (l == 0) then
         write(*,*)'Wrong separator in vertices.dat. It should be ":"'
         stop
        end if
        proc=cline(1:l-1)
        do 
         n=index(proc,space)
         if (n > 1) then
          j=j+1
          prtcl(j)=proc(1:n-1)
          if (iewk == 1) then
           do i=1,ipmod
            if (trim(prtcl(j)) == trim(cprt(i,1))) then
             if (iprt(i,5) == 21) then
              idscrd=1
              exit
             end if
            end if
           end do
          end if
          if (ihgs == 1) then
           do i=1,ipmod
            if (trim(prtcl(j)) == trim(cprt(i,1))) then
             if (iprt(i,5) == 25) then
              idscrd=1
              exit
             end if
            end if
           end do
          end if
         else
          exit
         end if
         if (idscrd == 1) go to 1001
         proc=adjustl(proc(n:))
        end do
        if (j == 4) then
         nbbbba=nbbbba+1
         if (nbbbba > nbbbb) then
          write(*,*)'Too many quartic couplings in vertices.dat.'//
     &              ' Increase nbbbb in maxvals.f.'
         end if
         do k=1,4
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))) then
            bbbb(nbbbba,k)=iprt(i,5)
            exit
           else if (trim(prtcl(k)) == trim(cprt(i,2))) then
            bbbb(nbbbba,k)=-iprt(i,5)
            exit
           else
            if (i == ipmod) then
             write(*,*)'readpart.f: No such particle in modpart.f'
             stop
            end if
           end if
          end do
         end do
         gbbbb(nbbbba)=trim(adjustl(cline(l+1:)))
        else if (j == 3) then
         ifrmn=0
         do k=1,3
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))
     &         .or. trim(prtcl(k)) == trim(cprt(i,2))) then
            if (iprt(i,2) /= 0) then
             ifrmn=1
             exit
            end if
           end if
          end do
          if (ifrmn == 1) exit
         end do
         if (ifrmn == 1) then
          nffba=nffba+1
          if (nffba > nffb) then
           write(*,*)'Too many fermion-fermion-boson couplings'//
     &               ' in vertices.dat.'
           write(*,*)'Increase nffb in maxvals.f.'
          end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             ffb(nffba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             ffb(nffba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gffb(nffba)=trim(adjustl(cline(l+1:)))
         else
          nbbba=nbbba+1
         if (nbbba > nbbb) then
          write(*,*)'Too many triple-boson couplings in vertices.dat.'
          write(*,*)'Increase nbbb in maxvals.f.'
         end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             bbb(nbbba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             bbb(nbbba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gbbb(nbbba)=trim(adjustl(cline(l+1:)))
         end if
        else
         write(*,*)'readpart.f: Only triple and quartic couplings'//
     &             ' are possible'
         stop
        end if
       end if
      end do

 700  close(66)

c      do i=1,nffba
c       write(*,*)'ffb(',i,':)=(/',ffb(i,:),'/)'
c       write(*,*)'gffb(',i,')=',gffb(i)
c      end do
c      do i=1,nbbba
c       write(*,*)'bbb(',i,':)=(/',bbb(i,:),'/)'
c       write(*,*)'gbbb(',i,')=',gbbb(i)
c      end do
c      do i=1,nbbbba
c       write(*,*)'bbbb(',i,':)=(/',bbbb(i,:),'/)'
c       write(*,*)'gbbbb(',i,')=',gbbbb(i)
c      end do

c      if (iewk == 1) then ! pure EW contributions
c       ffb(1:6,:)=0
c       bbb(1,:)=0
c       bbbb(1,:)=0
c      end if

      do i=1,nffba
       do j=1,3
        do k=1,ipmod
         if (abs(ffb(i,j)) == iprt(k,5)) then
          cffb(i,j)=iprt(k,3)
          mffb(i,j)=cprt(k,3)
          wffb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do


      do i=1,nbbba
       do j=1,3
        do k=1,ipmod
         if (abs(bbb(i,j)) == iprt(k,5)) then
          cbbb(i,j)=iprt(k,3)
          mbbb(i,j)=cprt(k,3)
          wbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbbbba
       do j=1,4
        do k=1,ipmod
         if (abs(bbbb(i,j)) == iprt(k,5)) then
          cbbbb(i,j)=iprt(k,3)
          mbbbb(i,j)=cprt(k,3)
          wbbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do




      nhel=1
c      nhip=1
      ncip=1

      do j=1,next
       nhel=nhel*f(j)%n
       if (f(j)%o == -1) then
c        nhip=nhip*f(j)%n
        ncip=ncip*f(j)%ct
       end if
      end do

      open(16,file='partcls.tmp',access='direct',form='formatted',
     &                                                     recl=80)
c      open(20,file='orders.tmp',access='direct',form='formatted',
c     &                                                     recl=80)

      do j=1,next
       write(16,1612,rec=j) f(j)%j,f(j)%i,f(j)%o,f(j)%g,f(j)%f,f(j)%ct,
     &                      f(j)%n,f(j)%e
c       write(20,1620,rec=j) j
      end do
 1612 format(6(i3),i24,i3)
 1620 format(26(i3))

      close(16)
c      close(20)


      write(*,'(a)') 'Attempting the process'
      write(*,'(a)') ' '
      write(*,'(a)') '        '//trim(adjustl(process))
      write(*,'(a)') ' '
      if (iewk == 1) then ! pure EW contributions
       write(*,'(a)') 'taking into account pure EW contributions'
      end if
      if (ihgs == 1) then ! No Higgs boson contributions
       write(*,'(a)') 'without the Higgs boson contributions'
      end if
      if (imcs == 1) then
       write(*,'(a)') 'MC summing over polarizations implemented'
      end if
      write(*,'(a)') 'External particles:'
      do i=1,next
       write(*,1) f(i)%ix,expc(i),f(i)%j,f(i)%i,f(i)%o,
     &            f(i)%o*f(i)%i*f(i)%j,f(i)%n
      end do
 1    format(i4,'  ',a3,i4,i4,i4,i4,i4)


      end
